
# q-expansion of newform 3536.2.a.e, downloaded from the LMFDB on 23 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3536
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-4], [2], [2], [-1], [1], [0], [4], [2], [2], [0], [0], [-4], [8], [-6], [-8], [-2], [-16], [14], [-16], [-8], [12], [-18], [-4], [-2], [-16], [4], [4], [14], [8], [-8], [6], [-4], [6], [0], [2], [-2], [6], [-22], [12], [-18], [-16], [-16], [-12], [-8], [-20], [24], [-30], [-18], [-26], [-12], [12], [12], [-18], [24], [18], [20], [-6], [22], [8], [-26], [-20], [0], [14], [12], [4], [2], [-36], [10], [-14], [-12], [4], [-34], [-22], [36], [18], [32], [12], [10], [-12], [34], [-18], [34], [-20], [-12], [-20], [10], [18], [-40], [4], [30], [-2], [20], [14], [20], [22], [-18], [-12], [-28], [32], [-30], [-44], [22], [12], [6], [16], [-34], [-16], [-26], [24], [-34], [16], [-46], [44], [10], [34], [-16], [-6], [-12], [2], [-14], [-18], [26], [2], [30], [8], [-44], [8], [2], [36], [26], [-24], [-42], [-42], [-42], [38], [22], [-18], [6], [-26], [8], [48], [42], [-14], [-50], [-8], [-2], [-20], [20], [48], [-18], [44], [-28], [24], [36], [0], [12], [-6], [28], [-18], [2], [4], [-36], [30], [-38], [-24], [-34], [20], [-46], [24], [42], [8], [-2], [0], [-14], [34], [52], [-16], [-34], [12], [20], [-26], [38], [44], [-38], [-50], [24], [32], [-54], [22], [-66], [-4], [-34], [-28], [6], [-28], [22], [-52], [56], [24], [-20], [-10], [46], [56], [30], [32], [36], [-12], [60], [-10], [18], [28], [-28], [42], [-8], [56], [38], [2], [18], [34], [-2], [-42], [-46], [-12], [-22], [-70], [-24], [56], [-30], [-50], [8], [-24], [10], [-20], [28], [-32], [-20], [-60], [-56], [-30], [-52], [0], [38], [8], [-48], [-74], [38], [-56], [24], [38], [-16], [32], [-70], [34], [68], [-54], [-62], [78], [-38], [-64], [-36], [-52], [-26], [-12], [20], [-14], [-22], [30], [-6], [-38], [20], [6], [40], [14], [-36], [4], [42], [-2], [-36], [-56], [18], [40], [-22], [42], [-16], [-70], [68], [-32], [14], [-18], [52], [18], [-50], [34], [-42], [48], [-22], [8], [16], [86], [38], [36], [76], [20], [-52], [44], [54], [-88], [6], [-72], [-16], [6], [-4], [16], [50], [12], [-62], [68], [-10], [-20], [88], [30], [-56], [56], [-50], [62], [54], [-28], [-22], [-44], [-22], [-56], [60], [10], [58], [50], [-52], [-36], [-8], [42], [18], [64], [-16], [50], [20], [58], [-44], [-26], [6], [32], [16], [6], [-8], [-6], [66], [-48], [90], [-42], [-48], [-8], [90], [16], [70], [4], [-62], [24], [-64], [90], [-60], [8], [10], [54], [0], [-42], [-30], [0], [26], [-82], [-50], [-72], [86], [54], [88], [-46], [-66], [-12], [26], [-74], [-8], [12], [28], [72], [100], [84], [58], [-48], [-58], [0], [-6], [-56], [-34], [-52], [64], [-18], [-92], [22], [102], [-28], [18], [46], [54], [-20], [44], [38], [6], [90], [56], [48], [-48], [-58], [40], [-30], [-36], [-28], [-2], [64], [36], [-28], [20]]
